package cn.com.qmth.stmms.api.controller;

import cn.com.qmth.stmms.api.exception.ApiException;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.utils.DateUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.Date;

@Controller("coreApiController")
@RequestMapping("/api/core")
public class CoreController extends BaseApiController {

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamStudentService studentService;

    @RequestMapping(value = "/exam/save", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject save(HttpServletRequest request, @RequestParam(required = false) Integer id,
            @RequestParam(required = false) String code, @RequestParam String name,
            @RequestParam(required = false) String examTime) {
        JSONObject result = new JSONObject();
        ApiUser user = RequestUtils.getApiUser(request);
        if (id == null && code == null) {
            Exam exam = new Exam();
            exam.setName(exam.getName());
            exam.setExamTime(DateUtils.parseDate(examTime));
            exam = examService.save(exam);
            result.accumulate("id", exam.getId());
            result.accumulate("updateTime", DateUtils.formatDateTime(exam.getUpdateTime()));
            return result;
        }
        Exam exam = null;
        if (id != null) {
            exam = examService.findById(id);
        } else if (code != null) {
            exam = examService.findBySchoolAndCode(user.getSchoolId(), code);
        }
        if (exam != null && exam.getSchoolId().equals(user.getSchoolId())) {
            exam.setName(name);
            exam.setExamTime(DateUtils.parseDate(examTime));
            exam = examService.save(exam);
            result.accumulate("id", exam.getId());
            result.accumulate("updateTime", DateUtils.formatDateTime(exam.getUpdateTime()));
            return result;
        } else {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
    }

    @RequestMapping(value = "/exam/student/save", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject studentSave(HttpServletRequest request, @RequestParam Integer examId,
            @RequestParam String examNumber, @RequestParam String studentCode, @RequestParam String name,
            @RequestParam String college, @RequestParam String className, @RequestParam String teacher,
            @RequestParam String subjectCode, @RequestParam String subjectName,
            @RequestParam(required = false) String packageCode, @RequestParam(required = false) String paperType,
            @RequestParam(required = false) String examSite, @RequestParam(required = false) String examRoom) {
        Exam exam = examService.findById(examId);
        if (exam == null) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        ExamStudent student = studentService.findByExamIdAndExamNumber(examId, examNumber);
        if (student == null) {
            student = new ExamStudent();
            student.setExamId(examId);
            student.setExamNumber(examNumber);
            student.setPackageCode(packageCode);

            student.setSchoolId(exam.getSchoolId());
            student.setAbsent(false);
            student.setUpload(false);
            student.setException(false);
            student.setSliceCount(0);
            student.setSheetCount(0);
            student.setObjectiveScore(0d);
            student.setSubjectiveScore(0d);
        }
        student.setStudentCode(studentCode);
        student.setName(name);
        student.setSubjectCode(subjectCode);
        student.setSubjectName(subjectName);
        student.setCollege(college);
        student.setClassName(className);
        student.setTeacher(teacher);
        student.setPaperType(paperType);
        student.setExamSite(examSite);
        student.setExamRoom(examRoom);
        studentService.save(student);
        JSONObject result = new JSONObject();
        result.accumulate("updateTime", DateUtils.formatDateTime(new Date()));
        return result;
    }

    @RequestMapping(value = "/student/query", method = RequestMethod.POST)
    @ResponseBody
    public JSONArray studentQuery(HttpServletRequest request, @RequestParam String studentCode,
            @RequestParam(required = false) String subjectCode, @RequestParam(required = false) Date minExamTime) {
        JSONArray array = new JSONArray();
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setStudentCode(studentCode);
        query.setSubjectCode(subjectCode);
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            Exam exam = examService.findById(student.getExamId());
            if (minExamTime != null && minExamTime.after(exam.getExamTime())) {
                JSONObject obj = new JSONObject();
                obj.accumulate("examId", student.getExamId());
                obj.accumulate("examCode", StringUtils.trimToEmpty(exam.getCode()));
                obj.accumulate("examTime", DateUtils.pastDays(exam.getExamTime()));
                obj.accumulate("examNumber", student.getExamNumber());
                obj.accumulate("name", student.getName());
                obj.accumulate("subjectCode", student.getSubjectCode());
                obj.accumulate("subjectName", student.getSubjectName());
                obj.accumulate("examSite", StringUtils.trimToEmpty(student.getExamSite()));
                obj.accumulate("examRoom", StringUtils.trimToEmpty(student.getExamRoom()));
                array.add(obj);
            }
        }
        return array;
    }

    @RequestMapping(value = "/exam/student/score", method = RequestMethod.POST)
    @ResponseBody
    public JSONArray getScore(HttpServletRequest request, @RequestParam(required = false) Integer examId,
            @RequestParam(required = false) String examCode, @RequestParam(required = false) String examNumber,
            @RequestParam(required = false) String studentCode, @RequestParam(required = false) String subjectCode,
            @RequestParam(required = false) String college, @RequestParam(required = false) String className,
            @RequestParam(required = false) String teacher,
            @RequestParam(required = false, defaultValue = "1") Integer pageNumber,
            @RequestParam(required = false, defaultValue = "100") Integer pageSize) {
        if (examId == null && examCode == null) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        Exam exam = examService.findById(examId);
        if (exam == null) {
            ApiUser user = RequestUtils.getApiUser(request);
            exam = examService.findBySchoolAndCode(user.getSchoolId(), examCode);
        }
        if (exam == null) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        JSONArray array = new JSONArray();
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(exam.getId());
        query.setExamNumber(examNumber);
        query.setStudentCode(studentCode);
        query.setSubjectCode(subjectCode);
        query.setCollege(college);
        query.setClassName(className);
        query.setTeacher(teacher);
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            JSONObject obj = new JSONObject();
            obj.accumulate("examId", exam.getId());
            obj.accumulate("examCode", exam.getCode());
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("studentCode", student.getStudentCode());
            obj.accumulate("name", student.getName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("paperType", StringUtils.trimToEmpty(student.getPaperType()));
            obj.accumulate("status", getStatus(student));
            DecimalFormat df = new DecimalFormat("####.###");
            obj.accumulate("totalScore", df.format(student.getTotalScore()));
            obj.accumulate("objectiveScore", df.format(student.getObjectiveScore()));
            obj.accumulate("subjectiveScore", df.format(student.getSubjectiveScore()));
            obj.accumulate("objectiveScoreDetail", StringUtils.trimToEmpty(student.getObjectiveScoreList()));
            obj.accumulate("subjectiveScoreDetail", StringUtils.trimToEmpty(student.getSubjectiveScoreList()));
            array.add(obj);
        }
        return array;
    }

    private int getStatus(ExamStudent student) {
        if (student.isBreach()) {
            return 3;
        }
        if (!student.isUpload() || student.isAbsent() || student.isManualAbsent()) {
            return 2;
        }
        return 1;
    }
}
